
# q-expansion of newform 1350.3.d.n, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1350
weight = 3
poly_data = [1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 3, 0], 1], [[0, 1, 0, 1], 1], [[0, 3, 0, -3], 1]]

hecke_ring_character_values = [[1001, [-1, 0, 0, 0]], [1027, [1, 0, 0, 0]]]
aps_data = [[0, 0, -1, 0], [0, 0, 0, 0], [0, 0, 0, 0], [4, 0, 0, -1], [0, -1, 0, 0], [-5, 0, 0, 2], [0, -4, 3, 0], [8, 0, 0, -4], [0, 5, -6, 0], [0, -8, 12, 0], [8, 0, 0, 5], [-11, 0, 0, -6], [0, -12, -15, 0], [28, 0, 0, -4], [0, -5, 6, 0], [0, -8, 36, 0], [0, -17, -24, 0], [-19, 0, 0, 18], [-8, 0, 0, -21], [0, -27, 6, 0], [-56, 0, 0, -12], [-40, 0, 0, -9], [0, 14, -48, 0], [0, -12, 3, 0], [25, 0, 0, -32], [0, -16, 87, 0], [28, 0, 0, -21], [0, 17, -60, 0], [104, 0, 0, -20], [0, -16, 15, 0], [16, 0, 0, -45], [0, 1, -108, 0], [0, 36, 60, 0], [-28, 0, 0, -21], [0, 0, -99, 0], [80, 0, 0, 12], [16, 0, 0, -40], [-212, 0, 0, 7], [0, 53, -90, 0], [0, -92, -39, 0], [0, -65, 36, 0], [-157, 0, 0, 26], [0, -8, 228, 0], [16, 0, 0, 12], [0, 52, -24, 0], [116, 0, 0, -5], [-4, 0, 0, 40], [184, 0, 0, 4], [0, 9, 228, 0], [-181, 0, 0, 2], [0, -48, -3, 0], [0, -67, 42, 0], [-1, 0, 0, 20], [0, 23, -192, 0], [0, 56, -147, 0], [0, -5, -198, 0], [0, -4, -120, 0], [-160, 0, 0, -24], [238, 0, 0, 16], [0, 56, 276, 0], [-8, 0, 0, 4], [0, -60, 231, 0], [280, 0, 0, 15], [0, -83, -186, 0], [-152, 0, 0, 52], [0, 76, 204, 0], [104, 0, 0, -87], [-344, 0, 0, 64], [0, -120, -120, 0], [32, 0, 0, 72], [0, 12, -372, 0], [0, -19, -222, 0], [292, 0, 0, 24], [-158, 0, 0, 16], [-76, 0, 0, 95], [0, 125, 66, 0], [0, -92, 252, 0], [-11, 0, 0, 118], [0, 72, -339, 0], [-169, 0, 0, 80], [0, -50, -144, 0], [-229, 0, 0, -102], [0, -261, 18, 0], [463, 0, 0, -40], [200, 0, 0, 57], [0, 89, -156, 0], [0, -16, -15, 0], [-167, 0, 0, 84], [0, 24, 384, 0], [-188, 0, 0, 12], [0, -135, 48, 0], [0, -8, -144, 0], [136, 0, 0, 119], [0, -226, 48, 0], [-364, 0, 0, 96], [0, 56, 96, 0], [0, 12, 3, 0], [0, 96, -141, 0], [508, 0, 0, 48], [155, 0, 0, 10], [148, 0, 0, 72], [0, 0, 111, 0], [0, 103, 276, 0], [0, 212, 111, 0], [152, 0, 0, -113], [895, 0, 0, -12], [0, -192, 192, 0], [0, 112, 48, 0], [0, 114, -336, 0], [-433, 0, 0, 40], [-728, 0, 0, 27], [331, 0, 0, 90], [0, 60, 9, 0], [116, 0, 0, -183], [-112, 0, 0, -4], [0, 276, -213, 0], [-800, 0, 0, 57], [0, -67, -558, 0], [0, -160, -249, 0], [0, -152, 492, 0], [773, 0, 0, 66], [-671, 0, 0, -12], [0, 212, -69, 0], [0, 191, -168, 0], [-220, 0, 0, 105], [0, 320, -177, 0], [299, 0, 0, 86], [0, 69, 138, 0], [-236, 0, 0, -96], [19, 0, 0, -42], [128, 0, 0, -3], [0, 51, -174, 0], [8, 0, 0, 24], [229, 0, 0, -126], [0, 256, 393, 0], [-688, 0, 0, -164], [0, -268, -180, 0], [-380, 0, 0, -168], [0, -32, 396, 0], [0, 300, 300, 0], [-232, 0, 0, 179], [0, 320, -171, 0], [-644, 0, 0, -52], [0, 55, 276, 0], [-475, 0, 0, 30], [0, 219, -90, 0], [-77, 0, 0, -198], [0, 40, -288, 0], [-172, 0, 0, 60], [0, 320, 180, 0], [-989, 0, 0, -118], [0, 276, 432, 0], [-332, 0, 0, -83], [0, -157, -378, 0], [328, 0, 0, -60], [0, 21, -1146, 0], [332, 0, 0, -228], [0, 448, -207, 0], [151, 0, 0, -220], [0, 52, 657, 0], [0, -368, -180, 0], [0, -388, 264, 0], [-44, 0, 0, -240], [0, -161, 204, 0], [0, 216, 48, 0], [0, 461, 210, 0], [248, 0, 0, 96], [-635, 0, 0, -126], [887, 0, 0, -84], [0, 164, -504, 0], [0, -98, -768, 0], [968, 0, 0, -180], [0, -163, 654, 0], [-617, 0, 0, 164], [-940, 0, 0, -119], [0, 412, 564, 0], [1364, 0, 0, 80], [0, -116, 219, 0], [-152, 0, 0, -100], [-37, 0, 0, -278], [1432, 0, 0, -140], [0, -167, -864, 0], [283, 0, 0, -6], [0, -376, 396, 0], [0, 104, -84, 0], [0, -452, 396, 0], [466, 0, 0, 144], [856, 0, 0, 184], [-607, 0, 0, 224], [0, -277, 162, 0], [-1151, 0, 0, 232], [0, 305, 228, 0], [-1384, 0, 0, -221], [0, -108, -1260, 0], [0, -351, 240, 0], [0, 4, 120, 0], [143, 0, 0, 276], [-653, 0, 0, -126], [0, -148, -255, 0], [0, 208, -864, 0], [0, -260, -3, 0], [1436, 0, 0, -180], [541, 0, 0, 126], [977, 0, 0, -48], [0, -57, -48, 0], [0, 512, 660, 0], [1304, 0, 0, 176], [0, 271, -384, 0], [0, -48, -1164, 0], [-700, 0, 0, 269], [184, 0, 0, -56], [0, 320, 288, 0], [1408, 0, 0, 45], [0, -231, -1212, 0], [0, -229, 918, 0], [248, 0, 0, -408], [-20, 0, 0, 404], [0, 304, -732, 0], [0, 107, -30, 0], [0, -708, 291, 0], [77, 0, 0, -478], [284, 0, 0, 183], [0, 368, 201, 0], [-1352, 0, 0, -276], [0, 489, -708, 0], [443, 0, 0, 238], [0, 0, 1404, 0], [0, -405, -90, 0], [1684, 0, 0, 64], [0, -320, -540, 0], [307, 0, 0, 314], [884, 0, 0, 171], [200, 0, 0, -329], [0, 160, 648, 0], [-140, 0, 0, 81], [0, -229, -210, 0], [-847, 0, 0, 108], [0, -48, -1176, 0], [0, -240, -1008, 0], [0, 259, 198, 0], [0, -392, 972, 0], [-268, 0, 0, -324], [-560, 0, 0, 93], [227, 0, 0, 354], [0, 52, -1764, 0], [0, -99, -918, 0], [-1232, 0, 0, 145], [0, -73, 408, 0], [-424, 0, 0, 536], [0, -59, -714, 0], [2251, 0, 0, -18], [0, 548, -264, 0], [0, 67, 1290, 0], [329, 0, 0, 264], [0, -388, -765, 0], [0, -119, 1248, 0], [896, 0, 0, -384], [1660, 0, 0, 36], [0, 452, 180, 0], [583, 0, 0, 336], [1744, 0, 0, 283], [0, 672, 72, 0], [350, 0, 0, -432], [277, 0, 0, -166], [0, 332, -807, 0], [-688, 0, 0, -132], [0, -656, -585, 0], [0, 156, -396, 0], [-548, 0, 0, -36], [0, -156, -396, 0], [2411, 0, 0, -98], [-992, 0, 0, 279], [-2111, 0, 0, 108], [1520, 0, 0, 393], [415, 0, 0, 160], [-752, 0, 0, 468], [0, -160, -396, 0], [2318, 0, 0, -160], [-655, 0, 0, 512], [0, 569, -804, 0], [0, -781, -606, 0], [-2656, 0, 0, -117], [0, -155, -150, 0], [-1741, 0, 0, 174], [-512, 0, 0, 381], [0, -101, -258, 0], [433, 0, 0, 308], [0, 1036, 108, 0], [-1336, 0, 0, 276], [0, 260, 1863, 0], [0, -492, 867, 0], [0, 272, -36, 0], [0, 400, 843, 0], [0, 289, -744, 0], [-1208, 0, 0, -108], [0, -1148, -63, 0], [416, 0, 0, 635], [0, 0, -267, 0], [0, 240, -1500, 0], [-32, 0, 0, -300], [1009, 0, 0, 96], [0, -380, -600, 0], [-1372, 0, 0, -431]]
